#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import datetime
import logging
from datetime import timedelta

from pystockdb.db.schema.stocks import Symbol, Tag
from pystockdb.tools.base import DBBase


class CreateAndFillDataBase(DBBase):
    """
    Database installer
    """

    def __init__(self, arguments: dict, logger: logging.Logger):
        super(CreateAndFillDataBase, self).__init__(arguments, logger)
        self.logger = logger
        self.currencies = arguments['currencies']
        self.history = arguments['max_history']
        self.indices_list = arguments['indices']
        self.prices = arguments.get('prices', True)

    def build(self):
        """
        Starts the database installation
        :return:
        """
        # add indices and stocks to db
        if not self.indices_list:
            return 0
        if (
            not all(
                cur in [Tag.EUR, Tag.USD, Tag.RUB] for cur in self.currencies
            )
            and self.prices
        ):
            self.logger.warning(
                'Currency {} is not supported.'.format(self.currencies)
            )
            return -1
        self.add_indices_and_stocks(self.indices_list)

        # skip download prices
        if not self.prices:
            return 0

        # add historical data
        for currency in self.currencies:
            # stocks
            symbols = Symbol.select(
                lambda t: (
                    Tag.YAO in t.item.tags.name
                    and currency in t.item.tags.name
                )
                or Tag.IDX in t.item.tags.name
            )
            end = datetime.datetime.now()
            start = end - timedelta(days=self.history * 365)
            self.download_historicals(
                symbols,
                start=start.strftime('%Y-%m-%d'),
                end=end.strftime('%Y-%m-%d'),
            )
        return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import yfinance as yf
from pytickersymbols import PyTickerSymbols


class DataCrawler:
    """
    Simple client for financial data
    """

    def __init__(self, skip_fix_symbols=False):
        self.ticker_symbols = None
        if not skip_fix_symbols:
            self.ticker_symbols = PyTickerSymbols()

    def __get_symbol(self, symbol):
        if self.ticker_symbols is not None:
            symbol_fix = self.ticker_symbols.index_to_yahoo_symbol(symbol)
            if symbol_fix is not None:
                symbol = symbol_fix
        return symbol

    def get_series(self, symbol, period='1d', start=None, end=None):
        """
        Creates a series object by given index/stock symbol
        :param symbol: symbol dax, mdax, i7n usw.
        :param period: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max
        :return: series objects or None if no data present
        """
        y_ticker = yf.Ticker(self.__get_symbol(symbol))
        data = y_ticker.history(period=period)
        return DataCrawler.series_to_object(data)

    def get_series_stack(self, symbols, period='1d', start=None, end=None):
        """
        Creates a series object by given index/stock symbol
        :param symbols: list of symbols dax, mdax, i7n usw.
        :param period: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max
        :param start: Download start date string (YYYY-MM-DD) Default is
        1900-01-01
        :param start: Download start date string (YYYY-MM-DD) Default is now
        :return: series objects or None if no data present
        """
        symbols = [self.__get_symbol(symbol) for symbol in symbols]
        symbols_str = ' '.join(symbols)
        if start and end:
            datas = yf.download(tickers=symbols_str, start=start, end=end,
                                group_by='ticker')
        else:
            datas = yf.download(tickers=symbols_str, period=period,
                                group_by='ticker')
        data_dict = {}
        for symbol in symbols:
            if len(symbols) > 1:
                data_dict[symbol] = DataCrawler.series_to_object(
                    datas[symbol].dropna()
                )
            else:
                data_dict[symbol] = DataCrawler.series_to_object(
                    datas.dropna()
                )
        return data_dict

    @staticmethod
    def series_to_object(data):
        """
        Transforms the json response to sqlalchemy object.
        :param data: json response
        :return: sqlalchemy object
        """
        series_list = []

        for timestamp, series in data.iterrows():
            series_list.append(
                {
                    'open': series.Open,
                    'close': series.Close,
                    'high': series.High,
                    'low': series.Low,
                    'volume': int(series.Volume),
                    'date': timestamp.to_pydatetime()
                }
            )
        return series_list

    def get_last_price(self, stock_object):
        """

        :param stock_object:
        :return:
        """
        symbol = self.__get_symbol(stock_object)
        data = self.get_series(symbol, '1d')
        return data

    def get_last_price_stack(self, symbols):
        """

        :param stock_object:
        :return:
        """
        datas = self.get_series_stack(symbols, '1d')
        return datas



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import datetime
import hashlib
import json
import logging
from datetime import timedelta

from pony.orm import commit, db_session, select

from pystockdb.db.schema.stocks import (
    Data,
    DataItem,
    Item,
    Price,
    PriceItem,
    Tag,
    Symbol,
)
from pystockdb.tools.base import DBBase
from pystockdb.tools.fundamentals import Fundamentals
from pystockdb.tools import ALL_SYMBOLS


class UpdateDataBaseStocks(DBBase):
    """
    Update Tool for autotrader database
    """

    def __init__(self, arguments: dict, logger: logging.Logger):
        super(UpdateDataBaseStocks, self).__init__(arguments, logger)
        self.logger = logger
        self.db_args = arguments['db_args']
        self.symbols = arguments['symbols']
        self.history = arguments.get('max_history', 5)

    def build(self):
        """
        Starts the update process
        :return: nothing
        """
        if self.arguments.get('prices', False):
            self.update_prices()
        if self.arguments.get('fundamentals', False):
            self.update_fundamentals()
        return 0

    @db_session
    def update_prices(self):
        """Update all prices of stocks
        """
        # get symbols
        prices = list(select((max(p.date), p.symbol) for p in Price))
        if ALL_SYMBOLS not in self.symbols:
            price_filtered = []
            for symbol in set(self.symbols):
                if len(prices) == 0:
                    # download initial data
                    price_filtered.append(
                        [
                            datetime.datetime.now()
                            - timedelta(days=365 * self.history),
                            Symbol.get(name=symbol),
                        ]
                    )
                else:
                    for price in prices:
                        if symbol == price[1].name:
                            price_filtered.append(price)
            prices = price_filtered
        # create update dict
        update = {}
        for price in prices:
            max_date = (price[0] + timedelta(days=1)).strftime('%Y-%m-%d')
            if max_date in update:
                update[max_date].append(price[1])
            else:
                update[max_date] = [price[1]]
        # update
        for key in update:
            start = datetime.datetime.strptime(key, '%Y-%m-%d')
            end = datetime.datetime.now()
            if start.date() >= end.date():
                continue
            self.download_historicals(
                update[key],
                start=start.strftime('%Y-%m-%d'),
                end=end.strftime('%Y-%m-%d'),
            )
        commit()

    @db_session
    def update_fundamentals(self):
        """Updates all fundamentals of stocks
        """
        # At the moment Fundamental client supports only usd symbols
        stocks = list(
            select(
                (pit.stock, sym.name)
                for pit in PriceItem
                for sym in pit.symbols
                if (Tag.GOG in sym.item.tags.name)
                and (Tag.USD in sym.item.tags.name)
            )
        )
        # filter specific stocks if not all
        if ALL_SYMBOLS not in self.symbols:
            stocks_filtered = []
            for stock in stocks:
                for sym in self.symbols:
                    for st_sym in stock[0].price_item.symbols:
                        if sym == st_sym.name:
                            stocks_filtered.append(stock)
            stocks = stocks_filtered
        # create list of google symbols
        gog_syms = [sto[1] for sto in stocks]
        fundamentals = Fundamentals(base_url=Fundamentals.BASE_URL)
        tickers = fundamentals.get_ticker_ids(gog_syms)
        for ticker in tickers:
            self.logger.info(
                'Download fundamentals for {}'.format(tickers[ticker])
            )
            stock = [sto[0] for sto in stocks if sto[1] == ticker]
            if len(stock) != 1:
                self.logger.warning(
                    'Can not download fundamentals for {}'.format(ticker)
                )
                continue
            stock = stock[0]
            ica = fundamentals.get_income_analysis(tickers[ticker])
            ifc = fundamentals.get_income_facts(tickers[ticker])
            rec = fundamentals.get_recommendation(tickers[ticker])
            ble = fundamentals.get_balance(tickers[ticker])
            ico = fundamentals.get_income(tickers[ticker])
            csh = fundamentals.get_cash_flow(tickers[ticker])
            for val in [
                (ica, Tag.ICA),
                (ifc, Tag.ICF),
                (rec, Tag.REC),
                (ble, Tag.BLE),
                (ico, Tag.ICO),
                (csh, Tag.CSH),
            ]:
                # hash stock name, tag and data
                m = hashlib.sha256()
                m.update(val[1].encode('UTF-8'))
                m.update(stock.name.encode('UTF-8'))
                data_str = json.dumps(val[0])
                m.update(data_str.encode('UTF-8'))
                shahash = m.hexdigest()
                # only add data if not exist
                if Data.get(hash=shahash) is None:
                    tag = Tag.get(name=val[1])
                    obj = Data(
                        data=val[0],
                        hash=shahash,
                        data_item=DataItem(item=Item(tags=[tag])),
                    )
                    stock.data_items.add(obj.data_item)
        commit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
from datetime import datetime
from pony.orm import Database, PrimaryKey, Set, Required, Optional, Json


db = Database()


class Index(db.Entity):
    uid = PrimaryKey(int, auto=True)
    stocks = Set('Stock')
    price_item = Required('PriceItem')
    name = Required(str)


class Stock(db.Entity):
    id = PrimaryKey(int, auto=True)
    indexs = Set(Index)
    name = Optional(str)
    orders = Set('Order')
    price_item = Required('PriceItem')
    data_items = Set('DataItem')

    def get_data(self, key):
        """
        Returns company data by key
        :param key:
        :return:
        """
        data_item = DataItem.select(
            lambda d: key in d.item.tags.name and d.stock == self
        ).first()
        if data_item:
            for data in data_item.datas:
                return data.data
        return None

    @staticmethod
    def __recommendation(key, attr, datas):
        if attr == 'rating' and 'rating' in datas:
            return float(datas['rating'])
        if 'measures' in datas:
            for strut in datas['measures']:
                if 'attr' in strut and 'value' in strut and \
                   strut['attr'] == attr:
                    try:
                        return float(strut['value'])
                    except ValueError:
                        return -1
        return -1

    @staticmethod
    def __other(key, attr, datas, annual, quarter_diff):
        for idx, data in enumerate(datas):
            if quarter_diff == 0 or idx >= quarter_diff:
                if not annual or annual and 'annual' in data and \
                   data['annual']:
                    if 'struts' in data:
                        for strut in data['struts']:
                            if 'attr' in strut and 'value' in strut and \
                               strut['attr'] == attr:
                                try:
                                    return float(strut['value'])
                                except ValueError:
                                    return -1
        return -1

    def get_data_attr(self, key, attr, annual=False, quarter_diff=0):
        """
        Get company data by key and attr
        :param key:
        :param attr:
        :param annual:
        :param quarter_diff:
        :return:
        """
        datas = self.get_data(key)
        if datas is None:
            return None

        if key == 'recommendation':
            return self.__recommendation(key, attr, datas)
        else:
            return self.__other(key, attr, datas, annual, quarter_diff)


class Signal(db.Entity):
    id = PrimaryKey(int, auto=True)
    result = Required('Result')
    orders = Set('Order')
    item = Required('Item')
    data_item = Optional('DataItem')
    price_items = Set('PriceItem')


class Tag(db.Entity):
    YAO = 'yahoo'
    GOG = 'google'
    USD = 'USD'
    EUR = 'EUR'
    RUB = 'RUB'
    IDX = 'index'
    ICA = 'incomeanalysis'
    ICF = 'incomefacts'
    REC = 'recommendation'
    ICO = 'income'
    BLE = 'balance'
    CSH = 'cash'

    id = PrimaryKey(int, auto=True)
    name = Optional(str)
    items = Set('Item')
    type = Required('Type')


class Type(db.Entity):
    IND = 'industry'
    REG = 'region'
    SYM = 'symbol'
    CUR = 'currency'
    MSC = 'misc'
    FDM = 'fundamentals'
    FIL = 'filter'
    ICR = 'indicator'
    ARG = 'argument'

    id = PrimaryKey(int, auto=True)
    name = Required(str, unique=True)
    tags = Set(Tag)

    def add_tags(self, names):
        """Add multiple tags to item object

        Arguments:
            names {list} -- list of tag names
        """
        for name in names:
            self.tags.create(name=name)


class Result(db.Entity):
    id = PrimaryKey(int, auto=True)
    date = Required(datetime)
    value = Required(float)
    status = Required(int)
    signal = Optional(Signal)
    arguments = Set('Argument')


class Symbol(db.Entity):
    id = PrimaryKey(int, auto=True)
    name = Optional(str, unique=True)
    price_item = Required('PriceItem')
    prices = Set('Price')
    item = Required('Item')


class Price(db.Entity):
    id = PrimaryKey(int, auto=True)
    open = Required(float)
    close = Required(float)
    high = Required(float)
    low = Required(float)
    volume = Required(int, size=64)
    date = Required(datetime)
    symbol = Required(Symbol)


class Portfolio(db.Entity):
    id = PrimaryKey(int, auto=True)
    name = Required(str)
    orders = Set('Order')
    cash = Required(float)
    value = Required(float)
    invest = Required(float)


class Order(db.Entity):
    id = PrimaryKey(int, auto=True)
    portfolio = Required(Portfolio)
    stock = Required(Stock)
    signals = Set(Signal)
    size = Optional(int)
    price = Optional(float)


class Item(db.Entity):
    id = PrimaryKey(int, auto=True)
    signal = Optional(Signal)
    tags = Set(Tag)
    price_item = Optional('PriceItem')
    argument = Optional('Argument')
    data_item = Optional('DataItem')
    symbol = Optional(Symbol)

    def add_tags(self, tags):
        """Add multiple tags to item object

        Arguments:
            tags {list} -- list of Tags
        """
        for tag in tags:
            self.tags.add(tag)


class PriceItem(db.Entity):
    id = PrimaryKey(int, auto=True)
    item = Required(Item)
    stock = Optional(Stock)
    index = Optional(Index)
    symbols = Set(Symbol)
    signals = Set(Signal)


class Argument(db.Entity):
    id = PrimaryKey(int, auto=True)
    arg = Required(str)
    item = Required(Item)
    result = Required(Result)


class Data(db.Entity):
    id = PrimaryKey(int, auto=True)
    date = Required(datetime, default=datetime.now())
    data = Required(Json)
    hash = Required(str)
    data_item = Required('DataItem')


class DataItem(db.Entity):
    id = PrimaryKey(int, auto=True)
    item = Required(Item)
    datas = Set(Data)
    signal = Optional(Signal)
    stock = Optional(Stock)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import logging

from pony.orm import db_session, select

from pystockdb.db.schema.stocks import Index
from pystockdb.tools.create import CreateAndFillDataBase


class SyncDataBaseStocks(CreateAndFillDataBase):
    """
    Sync Tool for autotrader database
    """

    def __init__(self, arguments: dict, logger: logging.Logger):
        # create db is not allowed during sync
        if 'db_args' in arguments and 'create_db' in arguments['db_args']:
            arguments['db_args']['create_db'] = False
        super(SyncDataBaseStocks, self).__init__(arguments, logger)

    def build(self):
        """
        Starts the update process
        :return: nothing
        """
        self.__set_indices()
        super(SyncDataBaseStocks, self).build()

    @db_session
    def __set_indices(self):
        indices_db = list(select(ind.name for ind in Index))
        self.indices_list = [ind for ind in self.indices_list
                             if ind not in indices_db]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
from setuptools import setup, find_packages
import re

EXCLUDE_FROM_PACKAGES = ['test', 'test.*', 'test*']
VERSION = '0.0.0'

with open("README.md", "r") as fh:
    long_description = fh.read()

INSTALL_REQUIRES = (
    [
        'pytickersymbols>=1.1.8',
        'pandas==1.2.3',
        'yfinance>=0.1.52',
        'uplink>=0.9.0',
        'pony==0.7.14'
    ]
)
with open('src/pystockdb/__init__.py', 'r') as fd:
    VERSION = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

setup(
    name="pystockdb",
    version=VERSION,
    author="Slash Gordon",
    author_email="slash.gordon.dev@gmail.com",
    package_dir={'': 'src'},
    description="Simple stock db with tools.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/portfolioplus/pystockdb",
    install_requires=INSTALL_REQUIRES,
    packages=find_packages('src', exclude=EXCLUDE_FROM_PACKAGES),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Office/Business :: Financial :: Investment',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import logging

from pony.orm import commit, db_session, core
from pytickersymbols import PyTickerSymbols

from pystockdb.db.schema.stocks import (
    Index,
    Item,
    PriceItem,
    Stock,
    Symbol,
    Tag,
    Type,
    db,
)
from pystockdb.tools.data_crawler import DataCrawler


class DBBase:
    """
    Database installer
    """

    def __init__(self, arguments: dict, logger: logging.Logger):
        self.logger = logger
        self.db_args = arguments["db_args"]
        self.arguments = arguments
        self.ticker_symbols = PyTickerSymbols()
        # has connection
        try:
            db.bind(**self.db_args)
        except core.BindingError:
            pass
        else:
            db.generate_mapping(check_tables=False)

        if self.arguments.get('create', False):
            db.drop_all_tables(with_all_data=True)
            db.create_tables()
            self.__insert_initial_data()

    def build(self):
        """
        Starts the database installation
        :return:
        """
        raise NotImplementedError

    @db_session
    def add_indices_and_stocks(self, indices_list):
        for index_name in indices_list:
            index = Index(name=index_name, price_item=PriceItem(item=Item()))
            # add index symbol
            yah_sym = self.ticker_symbols.index_to_yahoo_symbol(index_name)
            if yah_sym is None:
                self.logger.warning(
                    'Can not translate {} into yahoo symbol'.format(index_name)
                )
                continue
            idx_item = Item()
            idx_item.tags.add(Tag.get(name=Tag.IDX))
            index.price_item.symbols.create(name=yah_sym, item=idx_item)
            stocks = self.ticker_symbols.get_stocks_by_index(index.name)
            for stock_info in stocks:
                self.__add_stock_to_index(index, stock_info)
            commit()

    def __add_stock_to_index(self, index, stock_info):
        stock_in_db = Stock.get(name=stock_info['name'])
        if stock_in_db:
            self.logger.info(
                'Add stock {}:{} to index.'.format(
                    index.name, stock_in_db.name
                )
            )
            index.stocks.add(stock_in_db)
        else:
            self.logger.info(
                'Add stock {}:{} to db'.format(
                    index.name, stock_info[Type.SYM]
                )
            )
            # create stock
            stock = Stock(
                name=stock_info['name'], price_item=PriceItem(item=Item())
            )
            # add symbols
            yao = Tag.get(name=Tag.YAO)
            gog = Tag.get(name=Tag.GOG)
            usd = Tag.get(name=Tag.USD)
            eur = Tag.get(name=Tag.EUR)
            rub = Tag.get(name=Tag.RUB)
            for symbol in stock_info['symbols']:
                if Tag.GOG in symbol and symbol[Tag.GOG] != '-':
                    self.__create_symbol(
                        stock, Tag.GOG, gog, symbol, eur, usd, rub
                    )
                if Tag.YAO in symbol and symbol[Tag.YAO] != '-':
                    self.__create_symbol(
                        stock, Tag.YAO, yao, symbol, eur, usd, rub
                    )
            index.stocks.add(stock)
            # connect stock with industry and country
            # country
            name = stock_info['country']
            country = Tag.select(
                lambda t: t.name == name and t.type.name == Type.REG
            ).first()
            country.items.add(stock.price_item.item)
            # industry
            indus = stock_info['industries']
            industries = Tag.select(
                lambda t: t.name in indus and t.type.name == Type.IND
            )
            for industry in industries:
                industry.items.add(stock.price_item.item)

    @db_session
    def __create_symbol(
        self, stock, my_tag, my_tag_item, symbol, eur, usd, rub
    ):
        if my_tag in symbol and symbol[my_tag] != '-':
            cur = None
            if (
                symbol[my_tag].startswith('FRA:')
                or symbol[my_tag].endswith('.F')
                or symbol[my_tag].startswith('BME:')
                or symbol[my_tag].endswith('.MC')
            ):
                cur = eur
            elif (
                symbol[my_tag].startswith('NYSE:')
                or symbol[my_tag].startswith('OTCMKTS:')
                or symbol[my_tag].startswith('NASDAQ:')
                or ('.' not in symbol[my_tag] and ':' not in symbol[my_tag])
            ):
                cur = usd
            elif symbol[my_tag].startswith('MCX:') or symbol[my_tag].endswith(
                '.ME'
            ):
                cur = rub

            item = Item()
            if cur:
                item.add_tags([my_tag_item, cur])
            else:
                self.logger.warning(
                    'Currency detection for Symbol {} failed.'.format(
                        symbol[my_tag]
                    )
                )

            if Symbol.get(name=symbol[my_tag]):
                self.logger.warning(
                    'Symbol {} is related to more than one'
                    ' stock.'.format(symbol[my_tag])
                )
            else:
                stock.price_item.symbols.create(item=item, name=symbol[my_tag])

    @db_session
    def download_historicals(self, symbols, start, end):
        if not (start and end):
            return False
        crawler = DataCrawler()
        chunks = [symbols[x: x + 50] for x in range(0, len(symbols), 50)]
        for chunk in chunks:
            ids = [symbol.name for symbol in chunk]
            if ids is None:
                continue
            print(ids)
            series = crawler.get_series_stack(ids, start=start, end=end)
            for symbol in chunk:
                self.logger.debug(
                    'Add prices for {} from {} until {}.'.format(
                        symbol.name, start, end
                    )
                )
                for value in series[symbol.name]:
                    symbol.prices.create(**value)
            commit()
        return True

    @db_session
    def __insert_initial_data(self):
        # insert types
        region_type = Type(name=Type.REG)
        industry_type = Type(name=Type.IND)
        Type(name=Type.MSC).add_tags([Tag.IDX])
        Type(name=Type.SYM).add_tags([Tag.YAO, Tag.GOG])
        Type(name=Type.CUR).add_tags([Tag.USD, Tag.EUR, Tag.RUB])
        Type(name=Type.FDM).add_tags(
            [Tag.ICA, Tag.ICF, Tag.REC, Tag.ICO, Tag.BLE, Tag.CSH]
        )
        Type(name=Type.FIL)
        Type(name=Type.ICR)
        Type(name=Type.ARG)
        countries = self.ticker_symbols.get_all_countries()
        for country in countries:
            region_type.tags.create(name=country)
        industries = self.ticker_symbols.get_all_industries()
        for industry in industries:
            industry_type.tags.create(name=industry)
        commit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pystockdb

  Copyright 2019 Slash Gordon

  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import base64

from uplink import Consumer, Query, get, params, returns


class Fundamentals(Consumer):
    BASE_URL = base64.b64decode(
        'aHR0cDovL3NlY3VyaXRpZXNhcGkuc3RvY2tzNjY2LmNvbS8='
    ).decode('utf-8')

    @returns.json(key='list')
    @params({'hasNumber': 0, 'clientOrder': 3, 'queryNumber': 30})
    @get('/api/search/tickers5')
    def search(self, keys: Query):
        """
        Search for stock by name
        :param keys: stock name or search string
        :return: json data
        """

    def get_ticker_ids(self, symbols: Query):
        """
        Converts symbols to ticker ids
        :param symbols: a list of stock symbols [ETR:ADS,ETR:RIB]
        :return: ticker ids
        """
        chunks = [symbols[x:x + 50] for x in range(0, len(symbols), 50)]
        ticker_dict = {}
        for chunk in chunks:
            data = self.__get_ticker_ids(','.join(chunk))
            for item in data:
                key = '{}:{}'.format(item['sourceExchangeCode'],
                                     item['tickerSymbol'])
                ticker_dict[key] = item['tickerId']
        return ticker_dict

    @returns.json
    @get('/api/stocks/ticker/googleFinancial/tickerIdMapping')
    def __get_ticker_ids(self, symbols: Query):
        """get ticker ids raw. keep in mind that only 50 items are supported"""

    @returns.json
    @get('/api/securities/stock/{ticker_id}/recommendation')
    def get_recommendation(self, ticker_id):
        """
        Returns recommendation data for stock
        :param ticker_id: ticker id
        :return: json data
        """

    @returns.json
    @get('/api/securities/stock/{ticker_id}/statementsV2Detail')
    def get_income_facts(self, ticker_id):
        """
        Returns income data for stock
        :param ticker_id: ticker id
        :return: json data
        """

    @returns.json
    @get('/api/securities/stock/{ticker_id}/incomeAnalysis/crucial')
    def get_income_analysis(self, ticker_id):
        """
        Returns income analysis data for stock
        :param ticker_id: ticker id
        :return: json data
        """

    @returns.json
    @get('/api/securities/stock/{ticker_id}/compBrief')
    def get_company_brief(self, ticker_id):
        """
        Returns company brief data for stock
        :param ticker_id: ticker id
        :return: json data
        """

    def get_cash_flow(self, ticker_id):
        """Returns cash flow of given symbol

        :param ticker_id: ticker id of symbol
        :type ticker_id: str
        :return: cash flow in json format
        :rtype: dict
        """
        return self.get_sheet(ticker_id, 3)

    def get_balance(self, ticker_id):
        """Returns balance of given symbol

        :param ticker_id: ticker id of symbol
        :type ticker_id: str
        :return: balance in json format
        :rtype: dict
        """
        return self.get_sheet(ticker_id, 2)

    def get_income(self, ticker_id):
        """Returns income of given symbol

        :param ticker_id: ticker id of symbol
        :type ticker_id: str
        :return: income in json format
        :rtype: dict
        """
        return self.get_sheet(ticker_id, 1)

    @returns.json
    @get('/api/securities/stock/{ticker_id}/statementsV2Detail')
    def get_sheet(self, ticker_id, sheet_type: Query("type"),
                  query_number: Query('queryNumber') = 300):
        """
        Returns financial sheet(income, balance, cash flow) data for stock
        :param ticker_id: ticker id
        :param sheet_type: income=1, balance=2 and cashflow = 3
        :param query_number:
        :return:
        """
